var CaseSummary = function()
{
	var cs = {};
	var debug = 0;
	var TRUNCATE_SIZE = [-1, 30, 20, 15];

	cs.initPage = function()
	{
		Events.addEventHandler('showConfidential', 'change', cs.showConfidentialOnChange);
		Events.doOnEnter('caseName', cs.caseSummary);
		Buttons.addClickHandler('btnCaseSummary', cs.caseSummary);
		Buttons.addClickHandler('printButton', cs.goPrint);
		Buttons.addClickHandler('close', App.closeWindow);
		
		var caseName = Data.get('page-data', 'case-name');
		var isAdmin = Data.int('page-data', 'admin');
		var isPatientCurator = Data.int('page-data', 'patient-curator');

		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo';
		url += '&caseName='+encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case: ' + Strings.encodeTags(caseName));
		Wizard.asyncJsonRequest(url, cs.caseInfoLoaded);
	}
	
	cs.caseInfoLoaded = function(response)
	{
		Doc.show('all-info');
		
		var caseName = Data.get('page-data', 'case-name');
		var caseInfo = response.caseInfo;
		var site = response.site;
		var patient = response.patient;
		cs.initSections(response.sections);

		if (caseInfo)
		{
			// Use returned case name, as it may have been found from personal number
			if (!patient || patient.allCases.indexOf(caseName) == -1)
			{
				caseName = caseInfo.name;
				Data.set('page-data', 'case-name', caseName);
				// Update header text to use the potentially changed case name
				if (Doc.element('path-case-summary'))
				{
					Doc.element('path-case-summary').innerHTML = 'Case summary - ' + Strings.encodeTags(caseName);
				}
			}

			cs.addColumn('case.name', cs.asLink('SAMPLE', caseInfo));
			cs.addColumn('case.registrationDate', cs.asDate(caseInfo.registrationDate));
			cs.addColumn('case.tubeContentType', Strings.encodeTags(caseInfo.tubeContentType));
			cs.addColumn('case.mutationAnalysis', Strings.encodeTags(caseInfo.mutationAnalysis));
			cs.addColumn('case.site', site ? site.name : null);
			cs.addColumn('case.projectFocus', Strings.encodeTags(caseInfo.projectFocus));
			cs.addColumn('case.nofTubes', caseInfo.nofTubes);
			cs.addColumn('case.otherPathNote', Strings.encodeTags(caseInfo.otherPathNote));
			cs.addColumn('case.description', Strings.encodeTags(caseInfo.description));
		}
		else
		{
			cs.addColumn('case.name', cs.asNoInfo('No case information has been registered'));
			Doc.hide('case-details');
		}

		if (patient)
		{
			cs.addColumn('patient.name', cs.asLink('BIOSOURCE', patient));
			cs.addColumn('patient.registrationDate', cs.asDate(patient.registrationDate));
			cs.addColumn('patient.personalNumber', cs.warnIfMissing(Strings.encodeTags(patient.personalNumber)));
			cs.addColumn('patient.allFirstNames', cs.warnIfMissing(Strings.encodeTags(patient.allFirstNames)));
			cs.addColumn('patient.familyName', cs.warnIfMissing(Strings.encodeTags(patient.familyName)));
			cs.addColumn('patient.gender', Strings.encodeTags(patient.gender));
			cs.addColumn('patient.allCases', cs.asCaseList(patient.allCases, caseName));
		}
		else
		{
			cs.addColumn('patient.name', cs.asNoInfo('No patient information has been registered'));
			Doc.hide('patient-details');
		}

		var specimen = response.specimen;
		var numSpecimen = 0;
		if (specimen && specimen.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(specimen.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < specimen.length; i++)
			{
				var s = specimen[i];
				cs.addColumn('specimen.name', cs.asLink('SAMPLE', s));
				cs.addColumn('specimen.registrationDate', cs.asDate(s.registrationDate));
				cs.addColumn('specimen.specimenType', Strings.encodeTags(s.specimenType));
				cs.addColumn('specimen.storageBox', cs.asBioPlateLocation(s.bioWell));
				cs.addColumn('specimen.samplingDate', cs.warnIfMissing(cs.asDateTime(s.samplingDate)));
				cs.addColumn('specimen.viableTumourCellsPercent', s.viableTumourCellsPercent);
				cs.addColumn('specimen.plNumber', cs.warnIfMissing(Strings.encodeTags(s.plNumber)));
				cs.addColumn('specimen.pad', cs.warnIfMissing(Strings.encodeTags(s.pad)));
/*
				cs.addColumn('specimen.remainingQuantity', cs.asQuantity(s.remainingQuantity, ' mg', 1000));
				cs.addColumn('specimen.originalQuantity', cs.asQuantity(s.originalQuantity, ' mg', 1000));
*/
				cs.addColumn('specimen.nofSections', s.nofSections);
				cs.addColumn('specimen.operatorDeliveryComment', cs.truncate(s.operatorDeliveryComment, truncateAt));
				cs.addColumn('specimen.description', cs.truncate(s.description, truncateAt));
				cs.addColumn('specimen.startPlate', cs.asLink('BIOPLATE', s.startPlate));
				numSpecimen++;
			}
		}
		if (numSpecimen == 0)
		{
			cs.addColumn('specimen.name', cs.asNoInfo('No specimen information has been registered'));
			Doc.hide('specimen-details');
		}

		var rna = response.rna;
		if (rna && rna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(rna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < rna.length; i++)
			{
				var r = rna[i];
				cs.addColumn('rna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('rna.flag', r.flag ? cs.asFlagInfo(r.flag, 'flag.png') : null);
				cs.addColumn('rna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('rna.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('rna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
				cs.addColumn('rna.extractionDate', cs.asDate(r.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.extractionDate, 'RNA extraction: ' + r.name, 'thermometer.png'));
				cs.addColumn('rna.extractionProtocol', cs.asLink('PROTOCOL', r.extractionProtocol, truncateAt));
				cs.addColumn('rna.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' µg'));
				cs.addColumn('rna.originalQuantity', cs.asQuantity(r.originalQuantity, ' µg'));
/*
				cs.addColumn('rna.ndConc', cs.asQuantity(r.ndConc, ' ng/µl'));
*/
				cs.addColumn('rna.description', cs.truncate(r.description, truncateAt));
				cs.addColumn('rna.startPlate', cs.asLink('BIOPLATE', r.startPlate));
/*
				
				var hasQc = r.nofQc ? true : false;
				cs.addColumn('rna.qc', hasQc ? (r.qcDate ? cs.asDate(r.qcDate) : cs.asNoInfo('Waiting for Caliper...')) : 'No');
				cs.addColumn('rna.nofQc', hasQc ? r.nofQc : null);
				cs.addColumn('rna.qcProtocol', hasQc ? cs.asLink('PROTOCOL', r.qcProtocol, truncateAt) : null);
				cs.addColumn('rna.qcPlate', hasQc ? cs.asBioPlateLocation(r.qcPlate)+cs.asFileLink(r.qcPdf, 'pdffile.png') : null);
				cs.addColumn('rna.qcRqs', hasQc && r.qcRqs ? Meludi.formatNumber(r.qcRqs, null, 1) : null);
				cs.addColumn('rna.qcRin', hasQc && r.qcRin ? Meludi.formatNumber(r.qcRin, null, 1) : null);
				cs.addColumn('rna.qcDescription', hasQc ? cs.truncate(r.qcDescription, truncateAt) : null);
*/
			}
		}
		else
		{
			cs.addColumn('rna.name', cs.asNoInfo('No RNA information has been registered'));
			Doc.hide('rna-details');
		}

		var dna = response.dna;
		if (dna && dna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(dna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < dna.length; i++)
			{
				var d = dna[i];
				cs.addColumn('dna.name', cs.asLink('EXTRACT', d));
				cs.addColumn('dna.registrationDate', cs.asDate(d.registrationDate));
				cs.addColumn('dna.storageBox', cs.asBioPlateLocation(d.bioWell));
				cs.addColumn('dna.extractionDate', cs.asDate(d.extractionDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', d.extractionDate, 'DNA extraction: ' + d.name, 'thermometer.png'));
				cs.addColumn('dna.extractionProtocol', cs.asLink('PROTOCOL', d.extractionProtocol, truncateAt));
				cs.addColumn('dna.remainingQuantity', cs.asQuantity(d.remainingQuantity, ' µg'));
				cs.addColumn('dna.originalQuantity', cs.asQuantity(d.originalQuantity, ' µg'));
/*
				cs.addColumn('dna.ndConc', cs.asQuantity(d.ndConc, ' ng/µl'));
*/
				cs.addColumn('dna.description', cs.truncate(d.description, truncateAt));
				cs.addColumn('dna.startPlate', cs.asLink('BIOPLATE', d.startPlate));
			}
		}
		else
		{
			cs.addColumn('dna.name', cs.asNoInfo('No DNA information has been registered'));
			Doc.hide('dna-details');
		}

/*
		var mrna = response.mrna;
		if (mrna && mrna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(mrna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < mrna.length; i++)
			{
				var r = mrna[i];
				cs.addColumn('mrna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('mrna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('mrna.storageBox', cs.asBioPlateLocation(r.bioWell)+cs.asFileLink(r.platePdf, 'pdffile.png'));
				cs.addColumn('mrna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('mrna.cleanupDate', cs.asDate(r.cleanupDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.cleanupDate, 'mRNA cleanup: ' + r.name, 'thermometer.png'));
				cs.addColumn('mrna.usedQuantity', cs.asQuantity(r.usedQuantity, ' µg'));
				cs.addColumn('mrna.description', cs.truncate(r.description, truncateAt));
			}
		}
		else
		{
			cs.addColumn('mrna.name', cs.asNoInfo('No mRNA information has been registered'));
			Doc.hide('mrna-details');
		}

		var cdna = response.cdna;
		if (cdna && cdna.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(cdna.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < cdna.length; i++)
			{
				var r = cdna[i];
				cs.addColumn('cdna.name', cs.asLink('EXTRACT', r));
				cs.addColumn('cdna.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('cdna.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('cdna.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('cdna.synthesisDate', cs.asDate(r.synthesisDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.synthesisDate, 'cDNA synthesis: ' + r.name, 'thermometer.png'));
				cs.addColumn('cdna.description', cs.truncate(r.description, truncateAt));
			}
		}
		else
		{
			cs.addColumn('cdna.name', cs.asNoInfo('No cDNA information has been registered'));
			Doc.hide('cdna-details');
		}
		
		var lib = response.lib;
		if (lib && lib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(lib.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < lib.length; i++)
			{
				var r = lib[i];
				cs.addColumn('lib.name', cs.asLink('EXTRACT', r));
				cs.addColumn('lib.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('lib.storageBox', cs.asBioPlateLocation(r.bioWell));
				cs.addColumn('lib.result', (r.result && r.result != 'Successful') ? cs.asFailInfo(r.result, 'error.png') : null);
				cs.addColumn('lib.cleanupDate', cs.asDate(r.cleanupDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.cleanupDate, 'Library cleanup: ' + r.name, 'thermometer.png'));
				cs.addColumn('lib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
				cs.addColumn('lib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
				cs.addColumn('lib.barcode.name', r.barcode.name);
				cs.addColumn('lib.ca_size', cs.asQuantity(r.ca_size, ''));
				cs.addColumn('lib.molarity', cs.asQuantity(r.library_molarity_est, ' nM'));
				cs.addColumn('lib.adapterFrac', cs.asPercent(r.library_frac_adpt));
				cs.addColumn('lib.qubitconc', cs.asQuantity(r.qubitconc, ' ng/µl'));
				cs.addColumn('lib.description', cs.truncate(r.description, truncateAt));
			}
		}
		else
		{
			cs.addColumn('lib.name', cs.asNoInfo('No library information has been registered'));
			Doc.hide('lib-details');
		}

		var pooledLib = response.pooledlib;
		if (pooledLib && pooledLib.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(pooledLib.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < pooledLib.length; i++)
			{
				var r = pooledLib[i];
				cs.addColumn('pooledlib.name', cs.asLink('EXTRACT', r));
				cs.addColumn('pooledlib.registrationDate', cs.asDate(r.registrationDate));
				cs.addColumn('pooledlib.poolDate', cs.asDate(r.poolDate)+cs.asLabEnvLink('http://givare2.onk.lu.se/', r.poolDate, 'Pooled library pooling: ' + r.name, 'thermometer.png'));
				cs.addColumn('pooledlib.remainingQuantity', cs.asQuantity(r.remainingQuantity, ' ng', 0.001));
				cs.addColumn('pooledlib.originalQuantity', cs.asQuantity(r.originalQuantity, ' ng', 0.001));
				cs.addColumn('pooledlib.poolMolarity', cs.asQuantity(r.poolMolarity, ' nM'));
				cs.addColumn('pooledlib.poolConc', cs.asQuantity(r.poolConc, ' ng/µl'));
				cs.addColumn('pooledlib.description', cs.truncate(r.description, truncateAt));
			}
		}
		else
		{
			cs.addColumn('pooledlib.name', cs.asNoInfo('No pooled library information has been registered'));
			Doc.hide('pooledlib-details');
		}

		var sequencingRuns = response.sequencingRun;
		if (sequencingRuns && sequencingRuns.length > 0)
		{
			var truncateAt = TRUNCATE_SIZE[Math.min(sequencingRuns.length-1, TRUNCATE_SIZE.length)];
			for (var i = 0; i < sequencingRuns.length; i++)
			{
				var sr = sequencingRuns[i];
				var fc = sr.flowCell;
				cs.addColumn('sequencingRun.name', cs.asLink('DERIVEDBIOASSAY', sr));
				cs.addColumn('sequencingRun.flowCell', cs.asLink('PHYSICALBIOASSAY', fc));
				cs.addColumn('sequencingRun.result', (sr.result && sr.result != 'Successful') ? cs.asFailInfo(sr.result, 'error.png') : null);
				cs.addColumn('sequencingRun.clusterDate', cs.asDate(fc.clusterDate));
				cs.addColumn('sequencingRun.startDate', cs.asDate(sr.startDate));
				cs.addColumn('sequencingRun.endDate', cs.asDate(sr.endDate));
				cs.addColumn('sequencingRun.description', cs.truncate(sr.description, truncateAt));
			}
		}
		else
		{
			cs.addColumn('sequencingRun.name', cs.asNoInfo('No sequencing information has been registered'));
			Doc.hide('sequencingRun-details');
		}
*/

		var linkedItems = document.getElementsByClassName('linked-item');
		for (var itemNo = 0; itemNo < linkedItems.length; itemNo++)
		{
			Events.addEventHandler(linkedItems[itemNo], 'click', Items.itemOnClick);
		}
		
		var linkedFiles = document.getElementsByClassName('linked-file');
		for (var fileNo = 0; fileNo < linkedFiles.length; fileNo++)
		{
			Events.addEventHandler(linkedFiles[fileNo], 'click', Files.viewFileOnClick);
		}

/*		
		var linkedLabEnvs = document.getElementsByClassName('linked-labenv');
		for (var labEnvNo = 0; labEnvNo < linkedLabEnvs.length; labEnvNo++)
		{
			Events.addEventHandler(linkedLabEnvs[labEnvNo], 'click', LabEnvs.viewDataOnClick);
		}
*/
		
		var linkedCases = document.getElementsByClassName('linked-case');
		for (var caseNo = 0; caseNo < linkedCases.length; caseNo++)
		{
			Events.addEventHandler(linkedCases[caseNo], 'click', cs.caseSummary);
		}
	}
	
	cs.initSections = function(sections)
	{
		for (var s = 0; s < sections.length; s++)
		{
			var section = sections[s];

			var details = Doc.element(section.id+'-details');
			// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
			if (details && details.style.display != 'none')
			{
				var headerDiv = Doc.element(section.id+'-header');
				if (headerDiv)
				{
					Doc.addClass(headerDiv, 'link');
					headerDiv.title = 'Hide/show details';
					headerDiv.sectionId = section.id;
					Events.addEventHandler(headerDiv, 'click', cs.toggleDetails);
				}
				
				var sectionDiv = Doc.element(section.id+'-info');
				Doc.addOrRemoveClass(sectionDiv, 'hide-details', section.hiddenDetails);
			}
		}
	}

	cs.toggleDetails = function(event)
	{
		var sectionId = event.currentTarget.sectionId;
		var details = Doc.element(sectionId+'-details');
		// If the details section is 'forcibly' hidden (eg. no items of that type), ignore this call
		if (details.style.display == 'none') return;
		
		// Add 'hide-details' class to main <div section-info> if it is visible
		var section = Doc.element(sectionId+'-info');
		var isVisible = section.className.indexOf('hide-details') == -1;
		Doc.addOrRemoveClass(section, 'hide-details', isVisible);

		// Save to BASE setting
		var url = '../CaseSummary.servlet?ID='+App.getSessionId();
		url += '&cmd=SetSectionVisibility';
		url += '&section='+sectionId+'&hidden='+(isVisible ? 1 : 0);
		Wizard.asyncJsonRequest(url, null, 'POST');
	}

	cs.addColumn = function(id, value)
	{
		var tr = Doc.element(id);
		var td = document.createElement('td');
		if (tr.hasAddedColumns)
		{
			td.className = 'extra-column';
			tr.childNodes[tr.childNodes.length-1].className += ' fixed-column';
		}
		tr.hasAddedColumns = true;
		if (value != null)
		{
			td.innerHTML = value == '' ? '&nbsp;' : value;
			if (tr.className.indexOf('dynamic-column') >= 0)
			{
				tr.style.display = 'table-row';
			}
		}
		else
		{
			td.innerHTML = '&nbsp;';
		}
		tr.appendChild(td);
	}

	cs.asLink = function(itemType, item, maxLength)
	{
		var link = '';
		if (item)
		{
			var name = cs.truncate(item.name, maxLength);
			if (item.id)
			{
				link = '<span class="link linked-item" data-item-type="'+itemType+'" data-item-id="'+item.id+'" data-no-edit="'+(item.editable ? 0 : 1)+'">';
				link += name + '</span>';
			}
			else
			{
				link = name;
			}
		}
		return link;
	}
	
	cs.asFileLink = function(file, icon)
	{
		var link = '';
		if (file)
		{
			link = '<span class="link linked-file" data-file-id="'+file.id+'"><img src="../images/'+icon+'"></span>';
		}
		return link;
	}

	cs.asLabEnvLink = function(sensor, date, title, icon)
	{
		var link = '';
		// Check if BASE extension LabEnv is installed
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		if (homeLabEnv)
		{
			if (date)
			{
				if (sensor)
				{
					// Link to pop-up window with lab environment data for the chosen sensor and date
					link = '<span class="link linked-labenv" data-sensor="'+sensor+'" data-date="'+date+'" data-title="'+title+'"><img src="../images/'+icon+'"></span>';
				}
			}
		}
		return link;
	}

	cs.asCaseList = function(allCases, mainCase)
	{
		var html = '';
		for (var i = 0; i < allCases.length; i++)
		{
			var cse = allCases[i];
			var cse2 = Strings.encodeTags(cse);
			if (html != '') html += ', ';
			if (cse != mainCase)
			{
				html += '<span class="link linked-case" data-case-name="'+cse2+'" title="Show summary of case #'+cse2+'">'+cse2+'</span>';
			}
			else
			{
				html += '<b>' + cse2 + '</b>';
			}
		}
		return allCases.length > 1 ? html : null;
	}

	
	// Format value as a date
	cs.asDate = function(value)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = value.substr(0, 4) + '-' + value.substr(4, 2) + '-' + value.substr(6, 2);
		}
		return value;
	}

	// Format as date+time value
	cs.asDateTime = function(value, compareToDate)
	{
		if (!value) return '';
		if (value.length == 8)
		{
			value = cs.asDate(value);
		}
		else if (value.length == 13)
		{
			// If the compareToDate is the same day as the 'value' date, skip the date and replace with white-space
			if (compareToDate && value.substr(0, 8) == compareToDate.substr(0, 8))
			{
				value = '<span class="invisible">'+cs.asDate(value.substr(0, 8)) + '</span> ' + value.substr(9, 2) + ':'+value.substr(11, 2)
			}
			else
			{
				value = cs.asDate(value.substr(0, 8)) + ' ' + value.substr(9, 2) + ':'+value.substr(11, 2);
			}
		}
		return value;
	}

	// Style the value as a quantity with (optional) unit
	cs.asQuantity = function(value, unit, scale)
	{
		var result = '';
		if (value != null)
		{
			if (scale) value = value / scale;
			result = Meludi.formatNumber(value, unit, 2);
		}
		return result;
	}
	
	// Style the value as a percentage value
	cs.asPercent = function(value)
	{
		var result = '';
		if (value != null)
		{
			result = value + '%';
		}
		return result;
	}


	// Style the message as a warning message
	cs.asWarning = function(message)
	{
		message = '<span class="warning">' + message + '</span>';
		return message;
	}

	// Generate a 'Missing' warning if the value is missing
	cs.warnIfMissing = function(value)
	{
		var message = value ? value : cs.asWarning('Missing');
		return message;
	}
	
	// Style the message as a 'no information' message
	cs.asNoInfo = function(message)
	{
		message = '<span class="no-info">' + message + '</span>';
		return message;
	}
	
	// Style the message as a 'flag information' message
	cs.asFlagInfo = function(message, icon)
	{
		message = '<span class="flag-info">' + message + '&nbsp;' + '<img src="../images/'+icon+'"></span>';
		return message;
	}

	// Style the message as a 'fail information' message
	cs.asFailInfo = function(message, icon)
	{
		message = '<span class="fail-info">' + message + '&nbsp;' + '<img src="../images/'+icon+'"></span>';
		return message;
	}

	// A biowell as the location
	cs.asBioPlateLocation = function(well)
	{
		if (!well) return '';
		var plate = well.bioPlate;
		var text = cs.asLink('BIOPLATE', plate)  + ' ' + well.location;
		if (plate.storage)
		{
			var storage = plate.storage;
			var tmp = [];
			if (storage.name) tmp[tmp.length] = Strings.encodeTags(storage.name);
			if (storage.section) tmp[tmp.length] = 'section: ' +storage.section;
			if (storage.tray) tmp[tmp.length] = 'tray: ' +storage.tray;
			if (storage.position) tmp[tmp.length] = 'position: ' +storage.position;

			text = '<span class="more-info" title="' + tmp.join('; ')+'">' + text + ' </span>';
		}
		return text;
	}

	cs.truncate = function(value, maxLength)
	{
		if (!value) return value;
		if (maxLength > 2 && value.length > maxLength) 
		{
			var tmp = '<span class="truncated" title="'+value+'">'+Strings.encodeTags(value.substring(0, maxLength-2)) + '...</span>';
			value = tmp;
		}
		else
		{
			value = Strings.encodeTags(value);
		}
		return value;
	}

	cs.showConfidentialOnChange = function()
	{
		var frm = document.forms['meludi'];
		Doc.addOrRemoveClass('all-info', 'hide-confidential', !frm.showConfidential.checked);
	}

	cs.caseSummary = function(event)
	{
		var frm = document.forms['meludi'];
		var caseName = Data.get(event.currentTarget, 'case-name');
		if (!caseName)
		{
			caseName = frm.caseName.value;
			if (!caseName)
			{
				Forms.showNotification('caseName', 'Please enter a case id (&quot;ML&quot; + 5 digits)', null, null, 'left');
				return;
			}
		}
		var url = 'case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		url += '&pageType='+Data.get('page-data', 'page-type');
		location.href = url;
	}

	cs.goPrint = function()
	{
		var caseName = Data.get('page-data', 'case-name');
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.<br>';
		printNote += ' You may have to <i>scale down</i> to fit everything on the width of the page.';
		Meludi.openPrintWindow('all-content', 'Case summary - ' + Strings.encodeTags(caseName), 'portrait', printNote, '../', 'case_summary.css');
	}

	return cs;
}();

var LabEnvs = function()
{
	var labenvs = {};

	/**
		Event handler that calls 'labenv.jar/reports/labenvironmentdatabaseeventpopup.jsp'
		when clicking on the attached target. The sensor id should be stored
		in attribute 'data-sensor', the date in 'data-date',
		and the title in 'data-title'.
	*/
	labenvs.viewDataOnClick = function(event)
	{
		var homeLabEnv = Data.get('page-data', 'home-labenv');
		var title = Data.get(event.currentTarget, 'title');
		var sensor = Data.get(event.currentTarget, 'sensor');
		var date = Data.get(event.currentTarget, 'date');
		var url = homeLabEnv;
		url += '/reports/labenvironmentdatabaseeventpopup.jsp';
		url += '?ID='+App.getSessionId();
		url += '&pageTitle='+encodeURIComponent(title);
		url += '&labSensorUrl='+encodeURIComponent(sensor);
		url += '&originDate='+encodeURIComponent(date);
		Dialogs.openPopup(url, title, 820, 768);
	}
	
	return labenvs;
}();

Doc.onLoad(CaseSummary.initPage);
